/*****************************************************************************
** Function: ReadByte()
** Short:    Reads any double digit hex value and converts it into a number.
*/

static LONG ReadByte(STRING Str)
{
   LONG Value;
   if (!*Str) return(0);
   if ((*Str >= '0') AND (*Str <= '9')) Value = (*Str - '0') * 16;
   else if ((*Str >= 'A') AND (*Str <= 'F')) Value = ((*Str - 'A')+10) * 16;
   else if ((*Str >= 'a') AND (*Str <= 'f')) Value = ((*Str - 'a')+10) * 16;
   else Value = 0;
   Str++;
   if ((*Str >= '0') AND (*Str <= '9')) Value += (*Str - '0');
   else if ((*Str >= 'A') AND (*Str <= 'F')) Value += (*Str - 'A') + 10;
   else if ((*Str >= 'a') AND (*Str <= 'f')) Value += (*Str - 'a') + 10;
   return(Value);
}

/****************************************************************************
OBSOLETE - Use GetRenderInfo()
****************************************************************************/

static ERROR GET_AmtColours(struct Render *Self, LONG *Result)
{
   struct Screen *screen;

   DPrintF("@Get.AmtColours:","[Render] Obsolete - use GetRenderInfo()");

   if (AccessObject(Self->ScreenID, 2000, &screen) IS ERR_Okay) {
      *Result = screen->Bitmap->AmtColours;
      ReleaseObject(screen);
      return(ERR_Okay);
   }
   else {
      *Result = 0;
      return(ObjectError(Self, ERH_GetField, ERR_ExclusiveDenied));
   }
}

/****************************************************************************
Use GetRenderInfo() for getting the bits per pixel.  You may set the bitsperpixel
field for explicitly defining the BPP for render buffers.
****************************************************************************/

static ERROR GET_BitsPerPixel(struct Render *Self, LONG *Result)
{
   struct Screen *screen;

   DPrintF("@Get.BitsPerPixel:","[Render] Obsolete - use GetRenderInfo()");

   if (AccessObject(Self->ScreenID, 2000, &screen) IS ERR_Okay) {
      *Result = screen->Bitmap->BitsPerPixel;
      ReleaseObject(screen);
      return(ERR_Okay);
   }
   else {
      *Result = 0;
      return(ObjectError(Self, ERH_GetField, ERR_ExclusiveDenied));
   }
}

static ERROR SET_BitsPerPixel(struct Render *Self, LONG *Value)
{
   Self->prvBitsPerPixel = *Value;
   return(ERR_Okay);
}

/****************************************************************************
OBSOLETE - Use GetRenderInfo()
****************************************************************************/

static ERROR GET_BytesPerPixel(struct Render *Self, LONG *Result)
{
   struct Screen *screen;

   DPrintF("@Get.BytesPerPixel:","[Render] Obsolete - use GetRenderInfo()");

   if (AccessObject(Self->ScreenID, 2000, &screen) IS ERR_Okay) {
      *Result = screen->Bitmap->BytesPerPixel;
      ReleaseObject(screen);
      return(ERR_Okay);
   }
   else {
      *Result = 0;
      return(ObjectError(Self, ERH_GetField, ERR_ExclusiveDenied));
   }
}

/****************************************************************************

###FIELD###
Name:   Bitmap
Short:  Readable field that returns a pointer to a render object's drawable Bitmap.
Type:   *Bitmap
Status: Get

###DESCRIPTION###
<p>In order to draw graphics to a render object, it is necessary to obtain
a pointer to the drawable Bitmap.  To get the correct pointer, you need to
read this field while you have exclusive access to the render object.</p>

<p>Due to the way that the graphics system has been designed, the only time
that you can safely draw to a render object is through the action
notification scheme.  This means that you have to tell the render object
that you want to listen to the Draw action.  When the render object
receives a Draw action, it will notify all listeners by calling
ActionNotify() for every connected object.  When you receive notice, you can
obtain the Bitmap and draw to it safely.</p>

<p>In all other situations, trying to retrieve the Bitmap will most likely
result in a NULL value being returned.</p>

###END###

****************************************************************************/

static ERROR GET_Bitmap(struct Render *Self, struct Bitmap **Bitmap)
{
   if (Self->prvDrawableBitmap) {
      *Bitmap = Self->prvDrawableBitmap;
      return(ERR_Okay);
   }
   else {
      *Bitmap = NULL;
      return(ERR_FieldNotSet);
   }
}

/*****************************************************************************

###FIELD###
Name:   Buffer
Short:  If set to TRUE, buffering will be used when drawing a render object.
Type:   LONG|BOOLEAN
Status: Get/Set

###DESCRIPTION###
<p>The Buffer field can be set to a value of TRUE if you wish to enable
graphical buffering in a render object.  This feature must be used on
occasions where a graphical class needs read access to a render object's
graphical area (for instance, translucency and alpha blended effects require
read access).  Buffering can also be used to eliminate flickering if a large
amount of overlapped drawing is occurring.</p>

<p>Buffering can slow down drawing operations as it introduces an extra step
to the drawing process (instead of drawing directly to the display, graphics
objects will draw to a buffer, which is then blitted to the display on their
behalf).  On the other hand, buffering will almost always give dramatic speed
increases when extensive read operations are taking place.</p>

<p>It is possible to check whether or not a render object is using buffering
by reading this field, or by checking for the BUFFER bit in the
<a href="#Flags">Flags</a> field.</p>

###END###

*****************************************************************************/

static ERROR GET_Buffer(struct Render *Self, LONG *Value)
{
   if (Self->BufferID) *Value = TRUE;
   else *Value = FALSE;
   return(ERR_Okay);
}

static ERROR SET_Buffer(struct Render *Self, LONG *Value)
{
/* Buffering is no longer relevant as all render objects are buffered by
   default (if not directly, then by their parent).

   if (*Value IS TRUE)       Self->Flags |= RNF_BUFFER;
   else if (*Value IS FALSE) Self->Flags &= ~RNF_BUFFER;
   else return(ERR_Failed);
*/
   return(ERR_Okay);
}

/****************************************************************************

###FIELD###
Name:   Colour
Short:  String-based field for setting the background colour.
Type:   STRING
Status: Set

###DESCRIPTION###
<p>If the render object should have a plain background colour, set this field
to the colour value that you want to use.  The colour must be specified in
the standard format of '#RRGGBB' for hexadecimal or 'Red,Green,Blue' for
a decimal colour.</p>

<p>Render objects that do not have a set colour will not be cleared when
being drawn.  This means that the background will consist of 'junk' graphics
and it is your job to clear the background as you see fit.  Either way,
you have the power to choose the most appropriate drawing model to suit
your needs.</p>

<p>If you set the Colour and later want to turn the background colour off,
write a NULL value to the Colour field.  Note that writing to the Colour
field while a render object is on display will always cause a graphical
redraw.</p>

###SEE ALSO###
Field: ColourRGB

###END###

****************************************************************************/

static ERROR SET_Colour(struct Render *Self, STRING Colour)
{
   if ((!Colour) OR (!*Colour)) {
      Self->Flags &= ~RNF_CLEAR;
      return(ERR_Okay);
   }

   /*** Set the background colour ***/

   if (*Colour IS '#') {
      Colour++;
      Self->prvColourRGB.Red   = ReadByte(Colour);
      if (*Colour) Colour++; if (*Colour) Colour++;
      Self->prvColourRGB.Green = ReadByte(Colour);
      if (*Colour) Colour++; if (*Colour) Colour++;
      Self->prvColourRGB.Blue  = ReadByte(Colour);
   }
   else {
      Self->prvColourRGB.Red = StrToInt(Colour);
      while ((*Colour) AND (*Colour != ',')) Colour++; if (*Colour) Colour++;
      Self->prvColourRGB.Green = StrToInt(Colour);
      while ((*Colour) AND (*Colour != ',')) Colour++; if (*Colour) Colour++;
      Self->prvColourRGB.Blue = StrToInt(Colour);
   }

   Self->Flags |= RNF_CLEAR;

   if (Self->Flags & RNF_VISIBLE) Action(AC_Draw, Self, NULL);

   return(ERR_Okay);
}

/****************************************************************************

###FIELD###
Name:   ColourRGB
Short:  An RGB based field for manipulating the background colour.
Type:   RGB
Status: Get/Set

###DESCRIPTION###
<p>This RGB based field has the same properties as the Colour field but is
manipulated using an RGB structure.</p>

###SEE ALSO###
Field: Colour

###END###

****************************************************************************/

static ERROR GET_ColourRGB(struct Render *Self, struct RGB *RGB)
{
   RGB->Red   = Self->prvColourRGB.Red;
   RGB->Green = Self->prvColourRGB.Green;
   RGB->Blue  = Self->prvColourRGB.Blue;
   return(ERR_Okay);
}

static ERROR SET_ColourRGB(struct Render *Self, struct RGB *RGB)
{
   if (!RGB) {
      Self->Flags &= ~RNF_CLEAR;
      return(ERR_Okay);
   }

   Self->prvColourRGB.Red   = RGB->Red;
   Self->prvColourRGB.Green = RGB->Green;
   Self->prvColourRGB.Blue  = RGB->Blue;
   Self->Flags |= RNF_CLEAR;

   if (Self->Flags & RNF_VISIBLE) Action(AC_Draw, Self, NULL);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Drag
Short:  This object-based field is used to control the dragging of objects around the display.
Type:   OBJECTID
Status: Read/Write

###DESCRIPTION###
<p>Allowing the user to drag objects around the display is a feature that no
interface is complete without.  The Drag field provides you with an easy
way to set-up rendered areas that can be dragged around when the user clicks
and holds an area while moving the pointing device around the display.</p>

<p>To use, simply write this field with a reference to an external Render
object that is to be dragged when the user interacts with the object.  For
instance, if you create a window with a title-bar at the top, you would
set the Drag field of the title-bar to point to the object ID of the window.
If necessary, you can set the Drag field to point back to your render object
(this can be useful for creating icons and other small items).</p>

<p>To turn off dragging, simply write back to the field with a NULL
value.</p>

###END###

*****************************************************************************/

/*****************************************************************************

###FIELD###
Name:   FastResize
Short:  Enables fast resizing when set to TRUE.
Type:   LONG|BOOLEAN
Status: Get/Set

###DESCRIPTION###
<p>Setting the FastResize field to TRUE enables fast resizing, a feature
which can speed up the resizing process by eliminating any unnecessary
redrawing.</p>

<p>By default, when a render object is resized it will be completely redrawn,
so that all graphical objects dependent on the size of the drawable are
displayed correctly.  However, if the graphics inside the render object are
not dependent on its size then this process will only waste time, especially
if the rendered area is being shrunk.  By enabling the FastResize option, the
rendered area will never be redrawn when shrunk, and only exposed areas will
be redrawn if the rendered area is being enlarged.</p>

###END###

*****************************************************************************/

static ERROR GET_FastResize(struct Render *Self, LONG *Value)
{
   if (Self->Flags & RNF_FASTRESIZE) *Value = TRUE;
   else *Value = FALSE;
   return(ERR_Okay);
}

static ERROR SET_FastResize(struct Render *Self, LONG *Value)
{
   if (*Value IS TRUE)       Self->Flags |= RNF_FASTRESIZE;
   else if (*Value IS FALSE) Self->Flags &= ~RNF_FASTRESIZE;
   else return(ERR_Failed);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Flags
Short:  Special flags.
Prefix: RNF
Type:   LONG|FLAGS
Status: Read

###DESCRIPTION###
<p>The Flags field allows you to set special options for a render object.
Note that you must use the OR option when setting this field so that you do
not overwrite any existing flags.  Forgetting to do so will inevitably give
you some head-scratching problems!</p>

<p>Here are the available flag settings:</p>

<table border="1" bordercolordark="#000000" cellspacing="0" cellpadding="2" bordercolorlight="#000000" width="80%" align="center">
<tr><td class="tableheader" bgcolor="#eeeeee">Flag</td><td class="tableheader" bgcolor="#eeeeee">Description</td></tr>
<tr><td width="10%">AFTERCOPY</td><td>Read-only.  Indicates that after-copy mode has been enabled.</td></tr>
<tr><td width="10%">AUTOQUIT</td><td>When this flag is set the render object will send a quit message to its supporting task when and if the Close method is called.  This flag is typically used when a render object represents a core window for an application.</td></tr>
<tr><td width="10%">CLEAR</td><td>Read-only.  This flag is automatically set if the render object has been asked to clear its background.</td></tr>
<tr><td width="10%">DISABLED</td><td>This flag is set if the Disable action has been called on a render object.  Calling the Enable action will turn off the flag setting.</td></tr>
<tr><td width="10%">HASFOCUS</td><td>Read-only.  If set, this flag indicates that the render object currently has the focus.</td></tr>
<tr><td width="10%">HOST</td><td>If the render object is intended to host drawables from foreign processes, the host flag will be enabled.</td></tr>
<tr><td width="10%">INVISIBLE</td><td>This flag is turned on if the invisible option has been enabled (refer to the <a href="#Invisible">Invisible</a> field).  Note that invisibility in this sense means that the render object contains no drawable content.  It does not mean that the render object has no representation on the display.</td></tr>
<tr><td width="10%">NOVERTICAL</td><td>Turns off all vertical movement (applies to the Move action only).</td></tr>
<tr><td width="10%">NOHORIZONTAL</td><td>Turns off all horizontal movement (applies to the Move action only).</td></tr>
<tr><td width="10%">PERVASIVECOPY</td><td>This flag can be set in conjunction with after-copy mode.  It forces the after-copy support routine to copy graphics over the entire drawable area, rather than avoiding the graphics of child drawables.</td></tr>
<tr><td width="10%">PRECOPY</td><td>Read-only.  Indicates that pre-copy mode has been enabled.</td></tr>
<tr><td width="10%">REGION</td><td>Read/Init only.  This flag is set if the <a href="#Region">Region</a> option has been used.</td></tr>
<tr><td width="10%">SCROLLCONTENT</td><td>If set, the render object will send Move and MoveToPoint messages if the Scroll and ScrollToPoint actions are ever called on the object.</td></tr>
<tr><td width="10%">STICKTOBACK</td><td>This flag is turned on if the render object has been asked to stick to the back of its container (refer to the <a href="#StickToBack">StickToBack</a> field).</td></tr>
<tr><td width="10%">STICKTOFRONT</td><td>This flag is turned on if the render object has been asked to stick to the front of its container (refer to the <a href="#StickToFront">StickToFront</a> field).</td></tr>
<tr><td width="10%">STICKY</td><td>Set the STICKY flag if you wish to prevent the render object from responding to the Move action.</td></tr>
<tr><td width="10%">VISIBLE</td><td>If a render object is visible to the user, the VISIBLE flag will be set.  If the flag is not set, the render object is hidden.</td></tr>
<tr><td width="10%">VIDEO</td><td>Set this flag if you would like the render object's data to be managed in video.  While this can give some speed advantages, be warned that video based drawables are limited to write-only operations.</td></tr>
</table>

###END###

*****************************************************************************/

static ERROR SET_Flags(struct Render *Self, LONG *Value)
{
   /*** Retain private and read-only flags when the user sets this field ***/
   Self->Flags = (Self->Flags & (RNF_PRIVATE|RNF_READONLY)) | (Value[0] & (~(RNF_PRIVATE|RNF_READONLY)));

   if (Self->Head.ObjectFlags & NF_INITIALISED) UpdateRenderList(Self);

   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Frame
Short:  Manipulates the frame number that a render object has on display.
Type:   LONG
Status: Read/Write

###DESCRIPTION###
<p>Through clever use of the Frame field, the graphics of a render object
can be organised into individual frames that can be switched in-and-out of
the display by updating the current frame number.  To use the Frame
field effectively you need to use frame-enabled objects and tell them what
frames to group themselves into.  You will also need a controlling
object (e.g. an Animation object) or some other mechansim for manipulating
the frame number.  Here is an example that creates a basic animation based
on frames:</p>

<pre>
  &lt;render center width="20%" height="20%" colour="#a0a0a0"&gt;
    &lt;animation speed="1" framecount="2"/&gt;
    &lt;box width="10" height="10" border="#000000" center frame="1"&gt;
    &lt;ellipse width="10" height="10" center frame="2" border="#000000"&gt;
  &lt;/render&gt;
</pre>

<p>Almost all graphical objects support frames, and other frame-control
classes are available in the form of the Movement and OnClick classes, as
well as others.</p>

<p>Please note that to see the results of a new Frame setting, you will need
to redraw the render object.</p>

###SEE ALSO###
Class: Animation, Movement, OnClick

###END###

*****************************************************************************/

static ERROR SET_Frame(struct Render *Self, LONG *Value)
{
   if (*Value < 0) Self->Frame = 0;
   else Self->Frame = *Value;
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Hide
Short:  Set to TRUE to hide a render object from the display.
Type:   BOOLEAN
Status: Init

###DESCRIPTION###
<p>You can decide whether a render object should be hidden or not by writing
to this field prior to initialisation.  Just set the field to TRUE and the
render object will not show up on the display when you initialise it.</p>

<p>Note that after initialisation, the correct way to manage the visibility
of a render object is to use the Hide and Show actions.</p>

###END###

*****************************************************************************/

static ERROR SET_Hide(struct Render *Self, LONG *Value)
{
   if (Self->Head.ObjectFlags & NF_INITIALISED) {
      if (*Value IS TRUE) Action(AC_Hide, Self, NULL);
      else if (*Value IS FALSE) Action(AC_Show, Self, NULL);
      else return(ERR_Failed);
   }
   else {
      if (*Value IS TRUE) Self->Flags |= RNF_HIDE;
      else Self->Flags &= ~RNF_HIDE;
   }

   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   KeyOutput
Short:  Manages access to the keyboard when the user focusses on the drawable.
Type:   OBJECTID
Status: Read/Init

###DESCRIPTION###
<p>The KeyOutput field eases keyboard management by automatically subscribing
an object to the keyboard when the drawable has the user's focus.  This is
often useful when writing programs that open a single window and require
feedback from the keyboard.  The object normally referenced in the KeyOutput
field is your CurrentTaskID().  Key presses will be sent to the referenced
object  via the DataChannels action.</p>

<p>Key presses will stop being sent once the drawable loses the user's focus.</p>

###END###

*****************************************************************************/

/****************************************************************************

###FIELD###
Name:   Opacity
Short:  Affects the level of translucency applied to a render object.
Type:   FLOAT
Status: Get/Set

###DESCRIPTION###
<p>This field determines the translucency level of a drawable area.  The
default setting is 100%, which means that the drawable will be solid.  Any
other value that you set here will alter the impact of a drawable over its
destination area.  High values will retain the boldness of the graphics,
while low values can render it close to invisible.</p>

<p>Note: The translucent drawing routine works by drawing the render content
to its internal buffer first, then copying the graphics that are immediately
in the background straight over the top with an alpha-blending routine.  This
is not always ideal and you can sometimes get better results by using the
pre-copy option instead (see the Precopy field).</p>

<p>Please note that the use of translucency is realised at a significant cost
to the CPU's time.</p>

###END###

****************************************************************************/

static ERROR GET_Opacity(struct Render *Self, FLOAT *Value)
{
   *Value = Self->prvOpacity * 100 / 255;
   return(ERR_Okay);
}

static ERROR SET_Opacity(struct Render *Self, FLOAT *Value)
{
   if (*Value < 0) Self->prvOpacity = 0;
   else if (*Value >= 100) {
      Self->prvOpacity = 255;
      Self->Flags &= ~RNF_AFTERCOPY;
   }
   else {
      Self->prvOpacity = (Value[0] * 255) / 100;
      Self->Flags |= RNF_AFTERCOPY;
   }
   UpdateRenderList(Self);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Precopy
Short:  Enables the masking feature for render objects.
Type:   LONG|BOOLEAN
Status: Get/Set

###DESCRIPTION###
<p>If you need a render object to use masked or blended graphics, set this field.
Enabling the precopy option allows you to draw graphics into your render object
with all the graphics immediately behind the drawable pre-copied into the drawing
buffer.</p>

<p>Precopying is commonly used for creating icons, designer windows and alpha
blending effects.</p>

<p>If you are not going to draw graphics into the drawable area, you should
use the Invisible option to allow the background to show through.  Precopying
introduces an extra step in the drawing process which is realised at a cost
of CPU time.</p>

###SEE ALSO###
Fields: Invisible

###END###

*****************************************************************************/

static ERROR GET_Precopy(struct Render *Self, LONG *Value)
{
   if (Self->Flags & RNF_PRECOPY) *Value = TRUE;
   else *Value = FALSE;
   return(ERR_Okay);
}

static ERROR SET_Precopy(struct Render *Self, LONG *Value)
{
   if (*Value IS TRUE)       Self->Flags |= RNF_PRECOPY;
   else if (*Value IS FALSE) Self->Flags &= ~RNF_PRECOPY;
   else return(ERR_Failed);

   if (Self->Head.ObjectFlags & NF_INITIALISED) UpdateRenderList(Self);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Movement
Short:  Limits the movement of a render object to vertical or horizontal shifts.
Type:   LONG|FLAGS
Prefix: MOVE_
Status: Read/Set

###DESCRIPTION###
<p>You can limit the movement of a render object by setting the flags in the
Movement field.  By default, a render object is capable of moving in all
directions.  By altering this field, you can restrict the movement of a
render object to a specific direction.  There are three possible flag
combinations:<p>

<pre>
   HORIZONTAL           Horizontal movement only.
   VERTICAL             Vertical movement only.
   HORIZONTAL|VERTICAL  Permit movement in all directions.
</pre>

<p>This field is only effective in relation to the Move action, and it is
possible to circumvent the restrictions by setting the coordinate fields
directly.</p>

###SEE ALSO###
Fields: BottomLimit, LeftLimit, RightLimit, TopLimit

###END###

*****************************************************************************/

static ERROR SET_Movement(struct Render *Self, LONG *Flags)
{
   if (*Flags IS MOVE_HORIZONTAL) Self->Flags = (Self->Flags & RNF_NOHORIZONTAL) | RNF_NOVERTICAL;
   else if (*Flags IS MOVE_VERTICAL) Self->Flags = (Self->Flags & RNF_NOVERTICAL) | RNF_NOHORIZONTAL;
   else if (*Flags IS (MOVE_HORIZONTAL|MOVE_VERTICAL)) Self->Flags &= ~(RNF_NOVERTICAL|RNF_NOHORIZONTAL);
   UpdateRenderList(Self);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Region
Short:  Specifies whether or not a render object is acting as a simple region.
Type:   LONG|BOOLEAN
Status: Get/Set

###DESCRIPTION###
<p>Setting the Region field to a value of TRUE allows you to create render
objects that act as basic drawable regions.  Region based render objects are
identical to normal render objects in most respects, except they do not
participate in the layered drawing process.  This means that they are
susceptible to the influence of other graphics within their parent object,
because they are treated no differently to Box, Text or other graphical
class types.</p>

<p>Region based render areas are most useful when you can sacrifice
functionality for speed.  Buttons, menu content and title bars are often
implemented as regions because they are small and consist of only a small
group of graphical objects.</p>

<p>Depending on the arrangement of the region and parent object's graphics,
drawing around regions can cause flickering.  Use of the
<a href="#Buffer">Buffer</a> option can avoid this problem if it becomes an
issue.</p>

###END###

*****************************************************************************/

static ERROR GET_Region(struct Render *Self, LONG *Value)
{
   if (Self->Flags & RNF_REGION) *Value = TRUE;
   else *Value = FALSE;
   return(ERR_Okay);
}

static ERROR SET_Region(struct Render *Self, LONG *Value)
{
   if (Self->Head.ObjectFlags & NF_INITIALISED) return(ERR_Failed);
   else {
      if (*Value IS TRUE)       Self->Flags |= RNF_REGION;
      else if (*Value IS FALSE) Self->Flags &= ~RNF_REGION;
      else return(ERR_Failed);
      return(ERR_Okay);
   }
}

/*****************************************************************************

###FIELD###
Name:   Host
Short:  If this field is enabled, the render object will be able to host drawables from foreign processes.
Type:   LONG|BOOLEAN
Status: Get/Init

###DESCRIPTION###
<p>Setting the Host field to a value of TRUE allows you to create a render
object that can contain render objects from foreign processes.  This is
a must for creating desktop areas where tasks will want to open their own
windows.  Failure to use the host option correctly will cause failures when
foreign processes try to open drawables that connect with your task
space.</p>

<p>Note that setting the host option forces the system to allocate a graphics
buffer equal to the size of the rendered area.  All drawable children will
also be forced to allocate graphics buffers.  Because this can be memory
intensive, please do not use the host option lightly.</p>

###END###

*****************************************************************************/

static ERROR GET_Host(struct Render *Self, LONG *Value)
{
   if (Self->Flags & RNF_HOST) *Value = TRUE;
   else *Value = FALSE;
   return(ERR_Okay);
}

static ERROR SET_Host(struct Render *Self, LONG *Value)
{
   if (Self->Head.ObjectFlags & NF_INITIALISED) return(ERR_Failed);
   else {
      if (*Value IS TRUE) {
         Self->Flags |= RNF_HOST;
         Self->Flags &= ~(RNF_INVISIBLE|RNF_REGION);
      }
      else if (*Value IS FALSE) Self->Flags &= ~RNF_HOST;
      else return(ERR_Failed);
      return(ERR_Okay);
   }
}

/*****************************************************************************

###FIELD###
Name:   RouteFocus
Short:  Diverts render focussing through a specific path.
Type:   OBJECTID
Status: Read/Set

###DESCRIPTION###
<p>As the user manipulates the objects on the screen display, focus
messages are sent to individual objects to let them know when they have
received or have lost the focus.  If you are creating an interface with
complex focus management, then you may need to use the RouteFocus field
for some render objects.</p>

<p>The RouteFocus field alters the path that is normally taken when a
collection of objects receive focus messages.  To illustrate by example,
take the following hierarchy of render objects:</p>

<pre>
  SystemDisplay
    Render1
      Render1.1
      Render1.2
      Render1.3
        Render1.3.1
    Render2
      Render2.1
    Render3
</pre>

<p>Imagine that the SystemDisplay object currently has the focus,
then the user clicks on render object 1.3.1, which represents a button.  This
causes the primary focus to be immediately set to 1.3.1, then focus
notifications are sent to Render1.3 and Render1, because they are
immediate parents of 1.3.1. Notification is not sent to the
SystemDisplay, because prior to the user-click, the SystemDisplay object
already had the focus.  This particular example illustrates the normal
chain of events when a new object takes the focus.</p>

<p>However, this is not suitable if a render object needs to form part of
a focus chain when it lies outside of the normal hierachy.  Imagine that
Render2 represents a menu, and Render1 is a window.  Clicking on Render2
will cause Render1 to lose the focus, which is innapropriate
because the menu should be closely linked to the application represented
by the window.  To fix this, we set the RouteFocus field belonging to the
Render2 object to point to Render1.  By doing this, after Render2
receives the focus the algorithm will be diverted down a different path,
thus 'fooling' Render1 into believing that one of its immediate children
has the focus.</p>

<p>Focus routing is a rarely used feature, but can be vital in building
complex interfaces that do not follow a hierarchical GUI structure.  You can
see many examples of focus routing being used in Athene's menu system.</p>

###END###

*****************************************************************************/

static ERROR SET_RouteFocus(struct Render *Self, OBJECTID *Value)
{
   Self->RouteFocusID = *Value;

   if (Self->Head.ObjectFlags & NF_INITIALISED) {
      UpdateRenderList(Self);
   }

   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   StickToBack
Short:  Set to TRUE if you want a render object to stick to the back of its container.
Type:   LONG|BOOLEAN
Status: Set

###DESCRIPTION###
<p>If you want a render object to stick to the back of its container, set
this field to TRUE.  When a render object is stuck to the back, other objects
will not be allowed to move behind it when the MoveToBack action is called
on them.</p>

<p>Note that use of the StickToBack option is no guarantee that a render
object will remain at the very back of its container (there may be other
objects also using the StickToBack option).</p>

###SEE ALSO###
Fields:  Flags, StickToFront
Actions: MoveToBack, MoveToFront

###END###

*****************************************************************************/

static ERROR SET_StickToBack(struct Render *Self, LONG *Value)
{
   if (*Value IS TRUE) Self->Flags = (Self->Flags & ~RNF_STICKTOFRONT) | RNF_STICKTOBACK;
   else if (*Value IS FALSE) Self->Flags &= ~RNF_STICKTOBACK;
   if (Self->Head.ObjectFlags & NF_INITIALISED) UpdateRenderList(Self);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   StickToFront
Short:  Set to TRUE if you want a render object to stick to the front of its container.
Type:   LONG|BOOLEAN
Status: Set

###DESCRIPTION###
<p>If you want a render object to stick to the front of its container, set
this field to TRUE.  When a render object is stuck to the front, other objects
will not be allowed to move in front of it when the MoveToFront action is
called on them.</p>

<p>Note that use of the StickToFront option is no guarantee that a render
object will remain at the very front of its container (there may be other
objects also using the StickToFront option).</p>

###SEE ALSO###
Fields:  Flags, StickToBack
Actions: MoveToBack, MoveToFront

###END###

*****************************************************************************/

static ERROR SET_StickToFront(struct Render *Self, LONG *Value)
{
   if (*Value IS TRUE) Self->Flags = (Self->Flags & ~RNF_STICKTOBACK) | RNF_STICKTOFRONT;
   else if (*Value IS FALSE) Self->Flags &= ~RNF_STICKTOFRONT;
   if (Self->Head.ObjectFlags & NF_INITIALISED) UpdateRenderList(Self);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Sticky
Short:  Set this field to TRUE to prevent a render object from being moved around.
Type:   LONG|BOOLEAN
Status: Get/Set

###DESCRIPTION###
<p>When the Sticky field is set to TRUE, any attempts to Move the object
will fail, which causes the object to stick to its position.  This feature is
typically used on objects that should hold their position under most
circumstances, e.g. buttons and icons.</p>

<p>It is possible to circumvent the Sticky field by writing to the coordinate
fields directly.</p>

###END###

*****************************************************************************/

static ERROR GET_Sticky(struct Render *Self, LONG *Value)
{
   if (Self->Flags & RNF_STICKY) *Value = TRUE;
   else *Value = FALSE;
   return(ERR_Okay);
}

static ERROR SET_Sticky(struct Render *Self, LONG *Value)
{
   if (*Value IS TRUE)       Self->Flags |= RNF_STICKY;
   else if (*Value IS FALSE) Self->Flags &= ~RNF_STICKY;
   else return(ERR_Failed);

   if (Self->Head.ObjectFlags & NF_INITIALISED) UpdateRenderList(Self);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   Invisible
Short:  Defines a render object as being transparent (no drawable content).
Type:   LONG|BOOLEAN
Status: Get/Set

###DESCRIPTION###
<p>If you need a render object to be completely transparent, set this field.
This will make the render object invisible, causing the background of the
parent drawable to show through.  The render object will behave normally in
every other respect.</p>

<p>While it is possible to draw graphics to transparent objects, this only
works if you take the responsibility of graphics maintenance (in certain
cases you may need to manually tell the render object when to redraw for
instance).</p>

<p>Invisible objects are best used only in cases where you want to mount
a collection of render objects inside a clearly defined rectangular area and
have an advantage in maintaining low overhead.  If you need more advanced
handling for graphical transparent regions, set the Precopy field instead.</p>

###SEE ALSO###
Field: Precopy

###END###

*****************************************************************************/

static ERROR GET_Invisible(struct Render *Self, LONG *Value)
{
   if (Self->Flags & RNF_INVISIBLE) *Value = TRUE;
   else *Value = FALSE;
   return(ERR_Okay);
}

static ERROR SET_Invisible(struct Render *Self, LONG *Value)
{
   if (*Value IS TRUE)       Self->Flags |= RNF_INVISIBLE;
   else if (*Value IS FALSE) Self->Flags &= ~RNF_INVISIBLE;
   else return(ERR_Failed);

   if (Self->Head.ObjectFlags & NF_INITIALISED) UpdateRenderList(Self);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   UserFocus
Short:  Refers to the render object that has the current focus.
Type:   OBJECTID
Status: Get

###DESCRIPTION###
<p>This readable field will tell you the drawable object that has the current
user focus (within the context of the object hierarchy).  If no object has the
user's focus, this field will return a value of NULL.</p>

###END###

*****************************************************************************/

static ERROR GET_UserFocus(struct Render *Self, OBJECTID *Value)
{
   OBJECTID *focuslist;

   if (AccessMemory(RPM_FocusList, MEM_READ, &focuslist) IS ERR_Okay) {
      *Value = focuslist[0];
      ReleaseMemory(focuslist);
      return(ERR_Okay);
   }
   else {
      DPrintF("@Get.UserFocus()","[Render:%d]", Self->Head.UniqueID);
      return(ERR_AccessMemory);
   }
}

/*****************************************************************************

###FIELD###
Name:   Visible
Short:  Indicates the visibility of a render object.
Type:   LONG|BOOLEAN
Status: Get/Set

###DESCRIPTION###
<p>If you need to know if a render object is visible or hidden, you can read
this field to find out either way.  A TRUE value is returned if the object is
visible and FALSE is returned if the object is invisible.  Note that visibility
is subject to the properties of the container that the render object resides
in.  For example, if a render object is visible but is contained within a
render object that is invisible, the end result is that both objects are
actually invisible.</p>

<p>Visiblity is directly affected by the Hide and Show actions if you
wish to change the visiblity of a render object.</p>

###END###

*****************************************************************************/

static ERROR GET_Visible(struct Render *Self, LONG *Value)
{
   if (Self->Flags & RNF_VISIBLE) *Value = TRUE;
   else *Value = FALSE;
   return(ERR_Okay);
}

static ERROR SET_Visible(struct Render *Self, LONG *Value)
{
   if (*Value IS TRUE) Action(AC_Show, Self, NULL);
   else if (*Value IS FALSE) Action(AC_Hide, Self, NULL);
   else return(ERR_Failed);
   return(ERR_Okay);
}

/*****************************************************************************

###FIELD###
Name:   WindowHandle
Short:  Refers to a render object's window handle, if relevant.
Type:   LONG
Status: Get/Init

###DESCRIPTION###
<p>This field refers to the window handle of a render object, but only if
such a thing is relevant to the platform that the system is running on.
Currently, this field is only usable when creating a primary render object
within an X11 window manager or Microsoft Windows.</p>

<p>It is possible to set the WindowHandle field prior to initialisation
if you want a render object to be based on a window that already exists.</p>

###END###

*****************************************************************************/

static ERROR GET_WindowHandle(struct Render *Self, WINHANDLE *Value)
{
   *Value = Self->prvScreenWindow;
   return(ERR_Okay);
}

static ERROR SET_WindowHandle(struct Render *Self, WINHANDLE *Value)
{
   if (Self->Head.ObjectFlags & NF_INITIALISED) return(ERR_Failed);
   if (*Value) {
      Self->prvScreenWindow = *Value;
   }
   return(ERR_Okay);
}

