#ifndef GRAPHICS_PICTURE_H
#define GRAPHICS_PICTURE_H TRUE

/*
**  picture.h
**
**  (C) Copyright 1996-2003 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

#ifndef GRAPHICS_BITMAP_H
#include <pandora/graphics/bitmap.h>
#endif

/***************************************************************************
** Picture class.
*/

#define VER_PICTURE 1.0

typedef struct Picture {
   OBJECT_HEADER            /* Header */
   struct Bitmap *Bitmap;   /* Bitmap details */
   LONG   Flags;            /* Flags like IMG_REMAP */
   LONG   DisplayHeight;    /* Screen height */
   LONG   DisplayWidth;     /* Screen width */
   LONG   Quality;          /* Quality rating when saving image (0% low, 100% high) */
   struct Bitmap *Mask;     /* Monochrome bit mask or alpha channel */

   PRIVATE_FIELDS
   MEMORYID prvLocationMID;
   MEMORYID prvDescriptionMID;
   MEMORYID prvDisclaimerMID;
   OBJECTID prvBitmapID;
   OBJECTID prvMaskID;
   STRING   prvLocation;
   BYTE     prvAuthor[60];
   BYTE     prvCopyright[80];
   BYTE     prvTitle[50];
   BYTE     prvSoftware[30];
   STRING   prvDescription;
   STRING   prvDisclaimer;
   BYTE     prvHeader[32];
   OBJECTPTR prvFile;
} objPicture;

#define MT_CacheBitmap -1

/*****************************************************************************
** Picture Flags
*/

#define PCF_RESIZEX    0x00000001   /* Allow horizontal resizing */
#define PCF_RESIZEY    0x00000008   /* Allow vertical resizing */
#define PCF_NEW        0x00000010   /* Create the picture from scratch */
#define PCF_INTERLACE  0x00000020   /* The image is stored in interlaced format */
#define PCF_MASK       0x00000040   /* A mask has been generated for the image */
#define PCF_ALPHA      0x00000080   /* The image is accompanied by an alpha channel in the Mask structure */
#define PCF_BACKGROUND 0x00000100   /* The image has a background colour (defined in Bitmap->ColourRGB) */
#define PCF_QUERY      0x00000200   /* Do not load any file data on initialisation, just query it */
#define PCF_32BITALPHA 0x00000400   /* When loading: If there is an alpha mask with the image, force the image data to 32-bit alpha mode */
#define PCF_CACHE      0x00000800   /* Put the resulting bitmap in a global cache for optimised image loading */

#define PCF_RESIZE      (PCF_RESIZEX|PCF_RESIZEY)
#define PCF_TRANSPARENT PCF_BACKGROUND

#define IMG_RESIZEX    PCF_RESIZEX
#define IMG_RESIZEY    PCF_RESIZEY
#define IMG_NEW        PCF_NEW
#define IMG_INTERLACE  PCF_INTERLACE
#define IMG_MASK       PCF_MASK
#define IMG_ALPHA      PCF_ALPHA
#define IMG_BACKGROUND PCF_BACKGROUND
#define IMG_QUERY      PCF_QUERY
#define IMG_32BITALPHA PCF_32BITALPHA
#define IMG_CACHE      PCF_CACHE

#endif /* GRAPHICS_PICTURE_H */
